#include "cpufeature.h"

void real_cpuid(struct kvm_cpuid_entry *entry)
{
    asm volatile("cpuid"
		 : "=a" (entry->eax),
		   "=b" (entry->ebx),
		   "=c" (entry->ecx),
		   "=d" (entry->edx)
		 : "a" (entry->function));
}

void filter_cpuid(struct kvm_cpuid_entry *entry, int long_mode)
{
    switch (entry->function) {
    case 0x00000001:
	/* pv */
	entry->edx &= ~(1 << (X86_FEATURE_VME     %32));
	entry->edx &= ~(1 << (X86_FEATURE_DE      %32));
	entry->edx &= ~(1 << (X86_FEATURE_PSE     %32));

	entry->edx &= ~(1 << (X86_FEATURE_SEP     %32));
	entry->edx &= ~(1 << (X86_FEATURE_MTRR    %32));
	entry->edx &= ~(1 << (X86_FEATURE_PGE     %32));

	/* hvm */
	entry->ecx &= ~(1 << (X86_FEATURE_MWAIT   %32));
	entry->edx &= ~(1 << (X86_FEATURE_APIC    %32));
	entry->edx &= ~(1 << (X86_FEATURE_PSE36   %32));
	break;

    case 0x80000001:
	/* pv */
	entry->edx &= ~(1 << (X86_FEATURE_RDTSCP  %32));

	/* hvm */
	entry->ecx &= ~(1 << (X86_FEATURE_LAHF_LM %32));
	if (long_mode)
	    break;
	entry->edx &= ~(1 << (X86_FEATURE_LM      %32));
	entry->edx &= ~(1 << (X86_FEATURE_SYSCALL %32));
	break;
    }
}
